@@ -19,15 +19,19 @@ import sys, os, re
 from glob import glob
 from kook.utils import read_file, write_file
 
-release   = prop('release', '0.9.0')
+release   = prop('release', '0.10.0')
 package   = prop('package', 'Oktest')
 copyright = prop('copyright', "copyright(c) 2010-2011 kuwata-lab.com all rights reserved")
 license   = "MIT License"
-kook_default_product = 'unittest'
+kookbook.default = 'unittest'
 
-python = prop('python', 'python')
+python = prop('python', sys.executable)
 
 
+###
+### test
+###
+
 python_binaries = [
     ('2.4', '/opt/local/bin/python2.4'),
     #('2.5', '/opt/local/bin/python2.5'),
@@ -56,8 +60,9 @@ def _with_backup(filepath):
     return deco
 
 
-TEST_NAMES = ('oktest', 'helpers', 'tracer', 'spec', 'testdeco',
-              'doc', 'assertions', 'dummy', 'runner', 'util')
+TEST_NAMES = ('oktest', 'tracer', 'spec', 'testdeco',
+              'doc', 'assertions', 'dummy', 'runner', 'context', 'reporter',
+              'util', 'skip', 'todo', )
 test_names = [ os.path.basename(x).replace('_test.py', '')
                    for x in glob("test/*_test.py") ]
 assert set(TEST_NAMES) == set(test_names)
@@ -79,14 +84,13 @@ def task_unittest(c, *args, **kwargs):
 @ingreds('test/doc_test.py')
 @spices("-a: do with python from 2.4 to 3.2", "[testnames...]")
 def task_test(c, *args, **kwargs):
-    """test 'doctest', 'helpers', 'tracer', or 'doc'"""
+    """do test"""
     flag_all = bool(kwargs.get('a'))
     if flag_all:
         pairs = python_binaries
     else:
         ver = '%s.%s' % (sys.version_info[0:2])
-        bin = 'python' # or sys.executable
-        pairs = [ (ver, bin) ]
+        pairs = [ (ver, python) ]
     test_names = args or TEST_NAMES
     gvars = globals()
     funcs = []
@@ -162,22 +166,128 @@ def file_doc_test_py(c):
     pat = r'\n(Tracer\n=+\n.*?)\nHelpers Reference'
     m = re.compile(pat, re.S).search(cont)
     s = m.group(1)
+    #
+    rexp = re.compile(r'^(    )', re.M)
+    def fn(m, rexp=rexp):
+        console = m.group(1)
+        return "\n\n" + rexp.sub(r'\1#|', console) + "\n"
+    s = re.compile(r'\n\n(    \$ .*?\n)\n', re.M|re.S).sub(fn, s)
+    #
     s = re.compile(r'^(\S)', re.M).sub(r'#\1', s)
     s = re.compile(r'^( *)(\$ python)', re.M).sub(r'\1#\2', s)
+    s = re.compile(r'^([ \t]*)(\.\.\.)', re.M).sub(r'\1ok (0) == 1 #...', s)
+    s = re.compile(r'def _\(self\): \.\.\.').sub('def _(self): pass', s)
     s = re.sub(r'\{\{\*(.*?)\*\}\}', r'\1', s)
     #
     f = open(c.product, 'w')
     f.write(
         "import re\n"
         "from oktest import ok\n"
+        "import unittest\n"
+        "def skipIf(condition, reason):\n"
+        "    def deco(func):\n"
+        "        if not condition:\n"
+        "            return func\n"
+        "        def func2(*args, **kwargs):\n"
+        "            return reason\n"
+        "        return func2\n"
+        "    return deco\n"
         "\n"
-        "if True:\n"
-        "\n"
+        "try:\n"
+        "    _skipIf_bkup = getattr(unittest, 'skipIf', None)\n"
+        "    unittest.skipIf = skipIf\n"
+        "    del skipIf\n"
+        "#--------------------------------------------------\n"
     )
     f.write(s)
+    f.write(
+        "#--------------------------------------------------\n"
+        "finally:\n"
+        "    if _skipIf_bkup:\n"
+        "        unittest.skipIf = _skipIf_bkup\n"
+    )
     f.close()
 
 
+def _with_README_backup(fn):
+    fname = "README.txt"
+    try:
+        f = open(fname); s = f.read(); f.close()
+        os.rename(fname, fname + ".bkup")
+        s = s.replace("{{*", "").replace("*}}", "")
+        f = open(fname, 'w'); f.write(s); f.close()
+        fn()
+    finally:
+        os.rename(fname + ".bkup", fname)
+
+
+###
+### package, dist
+###
+
+text_files = ['README.txt', 'CHANGES.txt', 'MIT-LICENSE', 'Kookbook.py', 'MANIFEST', 'MANIFEST.in', 'setup.py']
+
+
+@recipe
+@ingreds('dist')
+def task_package(c):
+    """create package"""
+    ## setup
+    dir = c%"dist/$(package)-$(release)"
+    @pushd(dir)
+    def do():
+        #system(c%'$(python) setup.py sdist')
+        system(c%'$(python) setup.py sdist --force-manifest')
+        #system('python setup.py sdist --keep-temp')
+    cp(c%'$(dir)/MANIFEST', '.')
+
+
+@recipe
+def task_dist(c, *args, **kwargs):
+    """create Oktest-X.X.X/ directory"""
+    ## create dir
+    dir = c%"dist/$(package)-$(release)"
+    if os.path.isdir(dir):
+        rm_rf(dir)
+    mkdir(dir)
+    ## copy files
+    files = [ f for f in text_files if os.path.exists(f) ]
+    store(files, dir)
+    store('lib/**/*.py', 'test/*.py', dir)
+    ## edit files
+    replacer = [
+        (r'\$Package\$',   package),
+        (r'\$Release\$',   release),
+        (r'\$Copyright\$', copyright),
+        (r'\$License\$',   license),
+        (r'\$Package:.*?\$',    '$Package: %s $'   % package),
+        (r'\$Release:.*?\$',    '$Release: %s $'   % release),
+        (r'\$Copyright:.*?\$',  '$Copyright: %s $' % copyright),
+        (r'\$License:.*?\$',    '$License: %s $'   % license),
+    ]
+    edit(c%"$(dir)/**/*", exclude=[c%'$(dir)/Kookbook.py'], by=replacer)
+    replacer2 = [
+        (r'\{\{\*(.*?)\*\}\}', r'\1'),
+        (r'0\.0\.0', release),
+    ]
+    edit(c%"$(dir)/README.txt", by=replacer2)
+    ## MANIFEST
+    #@pushd(dir)
+    #def do():
+    #    rm_f('MANIFEST')
+    #    system(c%'$(python) setup.py sdist --force-manifest')
+
+
+kookbook.load('@kook/books/clean.py')
+CLEAN.extend(('**/*.pyc', '**/__pycache__', 'lib/*.egg-info', '%s.zip' % package))
+
+
+@recipe
+def task_manifest(c):
+    """update MANIFEST file"""
+    system(c%'$(python) setup.py sdist --force-manifest')
+
+
 def replacer(s):
     #s = re.sub(r'\$Package\$',   package,   s)
     #s = re.sub(r'\$Release\$',   release,   s)
@@ -205,83 +315,10 @@ def task_edit(c):
     edit('setup.py', by=repl)
 
 
-@recipe
-@ingreds('sdist', 'eggs')
-def task_package(c):
-    """create package"""
-    pass
-
-
-def _with_backup(fn):
-    fname = "README.txt"
-    try:
-        f = open(fname); s = f.read(); f.close()
-        os.rename(fname, fname + ".bkup")
-        s = s.replace("{{*", "").replace("*}}", "")
-        f = open(fname, 'w'); f.write(s); f.close()
-        fn()
-    finally:
-        os.rename(fname + ".bkup", fname)
-
-
-@recipe
-@ingreds('edit')
-def task_sdist(c, *args, **kwargs):
-    """create dist/Oktest-X.X.X.tar.gz"""
-    #rm_rf(c%"dist/$(package)-$(release)*")
-    @_with_backup
-    def fn():
-        system(c%'$(python) setup.py sdist')   # or setup.py sdist --keep-temp
-    #with chdir('dist') as d:
-    #    targz = c%"$(package)-$(release).tar.gz"
-    #    #tar_xzf(targz)
-    #    system(c%"tar xzf $(targz)")
-    #    dir = targz.replace('.tar.gz', '')
-    #    edit(c%"$(dir)/**/*", by=replacer)
-    #    mv(targz, c%"$(targz).old")
-    #    #tar_czf(c%"$(dir).tar.gz", dir)
-    #    system(c%"tar -cf $(dir).tar $(dir)")
-    #    system(c%"gzip -f9 $(dir).tar")
-
-
-def _do_setup_py(c, command):
-    for ver, bin in python_binaries:
-        system(c%command)
-        rm_rf(c%'lib/$(package).egg-info')
-
-
-@recipe
-@ingreds('edit')
-def task_eggs(c):
-    """create dist/*.egg files"""
-    @_with_backup
-    def fn():
-        _do_setup_py(c, "$(bin) setup.py bdist_egg")
-
-
-@recipe
-@ingreds('edit')
-def task_register(c):
-    """register information into PyPI"""
-    @_with_backup
-    def fn():
-        system(c%"$(python) setup.py register")
-
-
-@recipe
-@ingreds('edit')
-def task_upload(c):
-    """upload packages"""
-    @_with_backup
-    def fn():
-        system(c%"$(python) setup.py sdist upload")
-        _do_setup_py(c, "$(bin) setup.py bdist_egg upload")
-
-
-@recipe
-def task_clean(c):
-    rm_rf('**/*.pyc', '**/__pycache__', 'dist', 'build', 'lib/*.egg-info', c%'$(package).zip')
 
+###
+### website
+###
 
 @recipe
 @product('website/index.html')
@@ -294,6 +331,8 @@ def file_website_index_html(c):
     opts = '--stylesheet-path=style.css --link-stylesheet --strip-class=field --strip-class=field-name --strip-class=field-body'
     system(c%'rst2html.py $(opts) $(ingred) > $(product)')
     def f(s):
+        s = s.replace('0.0.0', release)
+        s = s.replace('$Release$', release)
         s = re.sub('<\?xml version=".*" encoding=".*" *\?>\n', '', s)
         s = s.replace('Oktest README', 'Oktest - a new style testing library -')
         s = s.replace('See CHANGES.txt', 'See <a href="CHANGES.txt">CHANGES.txt</a>')
@@ -308,13 +347,23 @@ def file_website_index_html(c):
         s = re.sub(pat, r, s)
         return s
     edit(c.product, by=f)
-
-
-@recipe
-def task_manifest(c):
-    """update MANIFEST file"""
-    system(c%'$(python) setup.py sdist --force-manifest')
-
+    #
+    def f2(s):
+        s = re.sub(r'<head>\n',
+                   ('<head>\n'
+                    '<meta http-equiv="Refresh" content="5;url=http://www.kuwata-lab.com/oktest/oktest-py_users-guide.html" />\n'
+                    ),
+                   s)
+        s = re.sub(r'<div class="document" id="oktest-readme">\n',
+                   ('<div class="document" id="oktest-readme">\n'
+                    '<div id="attention" style="border:solid 2px red;background:#FEE;font-size:large;padding:10px;margin-bottom:20px;">\n'
+                    '  <p><strong style="color:red">ATTENTION</strong></p>\n'
+                    '  <p>Oktest homepage is moved to <a href="http://www.kuwata-lab.com/oktest/">http://www.kuwata-lab.com/oktest/</a></p>\n'
+                    '</div><!-- /attention -->\n'
+                    ),
+                   s)
+        return s
+    edit(c.product, by=f2)
 
 @recipe
 @product('website/CHANGES.txt')
