@@ -3,13 +3,13 @@
 ###
 ### oktest.py -- new style test utility
 ###
-### $Release: 0.9.0 $
+### $Release: 0.10.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 
-__all__ = ('ok', 'NOT', 'NG', 'not_ok', 'run', 'spec', 'test')
-__version__ = "$Release: 0.9.0 $".split()[1]
+__all__ = ('ok', 'NOT', 'NG', 'not_ok', 'run', 'spec', 'test', 'fail', 'skip', 'todo', 'subject', 'situation', 'main')
+__version__ = "$Release: 0.10.0 $".split()[1]
 
 import sys, os, re, types, traceback, time, linecache
 
@@ -42,75 +42,16 @@ if python3:
     def _func_firstlineno(func):
         return func.__code__.co_firstlineno
 
-def _read_binary_file(fname):
-    f = open(fname, 'rb')
-    try:
-        b = f.read()
-    finally:
-        f.close()
-    return b
-if python2:
-    _rexp_magic_comment = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)')
-    def _read_text_file(fname):
-        b = _read_binary_file(fname)
-        m = _rexp_magic_comment.match(b)
-        encoding = m and m.group(1) or 'utf-8'
-        u = b.decode(encoding)
-        assert isinstance(u, unicode)
-        return u
-if python3:
-    _rexp_magic_comment = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)'.encode('us-ascii'))
-    def _read_text_file(fname):
-        b = _read_binary_file(fname)
-        m = _rexp_magic_comment.match(b)
-        encoding = m and m.group(1).decode('us-ascii') or 'utf-8'
-        u = b.decode(encoding)
-        assert isinstance(u, str)
-        return u
-
-def _get_location(depth=0):
-    frame = sys._getframe(depth+1)
-    return (frame.f_code.co_filename, frame.f_lineno)
-
-def _new_module(name, local_vars, helper=None):
+def _new_module(name, local_vars, util=None):
     mod = type(sys)(name)
     sys.modules[name] = mod
     mod.__dict__.update(local_vars)
-    if helper and getattr(mod, '__all__', None):
+    if util and getattr(mod, '__all__', None):
         for k in mod.__all__:
-            helper.__dict__[k] = mod.__dict__[k]
-        helper.__all__ += mod.__all__
+            util.__dict__[k] = mod.__dict__[k]
+        util.__all__ += mod.__all__
     return mod
 
-if python3:
-    def func_argnames(func):
-        if isinstance(func, types.MethodType):
-            codeobj = func.__func__.__code__
-            index = 1
-        else:
-            codeobj = func.__code__
-            index = 0
-        return codeobj.co_varnames[index:codeobj.co_argcount]
-    def func_defaults(func):
-        if isinstance(func, types.MethodType):
-            return func.__func__.__defaults__
-        else:
-            return func.__defaults__
-else:
-    def func_argnames(func):
-        if isinstance(func, types.MethodType):
-            codeobj = func.im_func.func_code
-            index = 1
-        else:
-            codeobj = func.func_code
-            index = 0
-        return codeobj.co_varnames[index:codeobj.co_argcount]
-    def func_defaults(func):
-        if isinstance(func, types.MethodType):
-            return func.im_func.func_defaults
-        else:
-            return func.func_defaults
-
 
 __unittest = True    # see unittest.TestResult._is_relevant_tb_level()
 
@@ -251,7 +192,7 @@ class AssertionObject(object):
     #    return self
 
     def failed(self, msg, depth=2, boolean=None):
-        file, line = _get_location(depth + 1)
+        file, line = util.get_location(depth + 1)
         diff = None
         if isinstance(msg, tuple):
             msg, diff = msg
@@ -525,24 +466,27 @@ ASSERTION_OBJECT = AssertionObject
 
 def ok(target):
     obj = ASSERTION_OBJECT(target, True)
-    obj._location = _get_location(1)
+    obj._location = util.get_location(1)
     return obj
 
 def NG(target):
     obj = ASSERTION_OBJECT(target, False)
-    obj._location = _get_location(1)
+    obj._location = util.get_location(1)
     return obj
 
 def not_ok(target):  # for backward compatibility
     obj = ASSERTION_OBJECT(target, False)
-    obj._location = _get_location(1)
+    obj._location = util.get_location(1)
     return obj
 
 def NOT(target):     # experimental. prefer to NG()?
     obj = ASSERTION_OBJECT(target, False)
-    obj._location = _get_location(1)
+    obj._location = util.get_location(1)
     return obj
 
+def fail(desc):
+    raise AssertionError(desc)
+
 
 class Should(object):
 
@@ -613,10 +557,40 @@ class SkipObject(object):
 skip = SkipObject()
 
 
+def todo(func):
+    def deco(*args, **kwargs):
+        exc_info = None
+        try:
+            func(*args, **kwargs)
+            raise _UnexpectedSuccess("test should be failed (because not implemented yet), but passed unexpectedly.")
+        except AssertionError:
+            raise _ExpectedFailure(sys.exc_info())
+    deco.__name__ = func.__name__
+    deco.__doc__  = func.__doc__
+    return deco
+
+class _ExpectedFailure(Exception):
+
+    def __init__(self, exc_info=None):
+        if exc_info:
+            self.exc_info = exc_info
+
+class _UnexpectedSuccess(Exception):
+    pass
+
+try:
+    from unittest.case import _ExpectedFailure, _UnexpectedSuccess
+except ImportError:
+    pass
+
+
+
 ST_PASSED  = "passed"
 ST_FAILED  = "failed"
 ST_ERROR   = "error"
 ST_SKIPPED = "skipped"
+ST_TODO    = "todo"
+#ST_UNEXPECTED = "unexpected"
 
 
 class TestRunner(object):
@@ -697,9 +671,37 @@ class TestRunner(object):
     def run_testcases(self, klass, testnames=None):
         if testnames is None:
             testnames = self.get_testnames(klass)
-        for testname in testnames:
-            testcase = self._new_testcase(klass, testname)
-            self.run_testcase(testcase, testname)
+        context_list = getattr(klass, '_context_list', None)
+        if context_list:
+            items = []
+            for tname in testnames:
+                meth = getattr(klass, tname)
+                if not hasattr(meth, '_test_context'):
+                    items.append((tname, meth))
+            items.extend(context_list)
+            TestContext._sort_items(items)
+            allowed = dict.fromkeys(testnames)
+            self._run_items(klass, items, allowed)
+        else:
+            for testname in testnames:
+                testcase = self._new_testcase(klass, testname)
+                self.run_testcase(testcase, testname)
+
+    def _run_items(self, klass, items, allowed):
+        for item in items:
+            if isinstance(item, tuple):
+                testname = item[0]
+                if testname in allowed:
+                    testcase = self._new_testcase(klass, testname)
+                    self.run_testcase(testcase, testname)
+            else:
+                assert isinstance(item, TestContext)
+                context = item
+                self._enter_testcontext(context)
+                try:
+                    self._run_items(klass, context.items, allowed)
+                finally:
+                    self._exit_testcontext(context)
 
     def _new_testcase(self, klass, method_name):
         try:
@@ -745,7 +747,15 @@ class TestRunner(object):
         except AssertionError:
             return ST_FAILED, sys.exc_info()
         except SkipTest:
-            return ST_SKIPPED, ()
+            return ST_SKIPPED, sys.exc_info()
+        except _ExpectedFailure:   # when failed expectedly
+            return ST_TODO, ()
+        except _UnexpectedSuccess: # when passed unexpectedly
+            #return ST_UNEXPECTED, ()
+            ex = sys.exc_info()[1]
+            if not ex.args:
+                ex.args = ("test should be failed (because not implemented yet), but passed unexpectedly.",)
+            return ST_FAILED, sys.exc_info()
         except Exception:
             return ST_ERROR, sys.exc_info()
         else:
@@ -766,6 +776,12 @@ class TestRunner(object):
     def _exit_testcase(self, testcase, testname, status, exc_info):
         self.reporter.exit_testcase(testcase, testname, status, exc_info)
 
+    def _enter_testcontext(self, context):
+        self.reporter.enter_testcontext(context)
+
+    def _exit_testcontext(self, context):
+        self.reporter.exit_testcontext(context)
+
     def __enter__(self):
         self.reporter.enter_all()
         return self
@@ -824,7 +840,9 @@ def run(*targets, **kwargs):
     finally:
         runner.__exit__(sys.exc_info())
     counts = runner.reporter.counts
-    return counts.get(ST_FAILED, 0) + counts.get(ST_ERROR, 0)
+    get = counts.get
+    #return get(ST_FAILED, 0) + get(ST_ERROR, 0) + get(ST_UNEXPECTED, 0)
+    return get(ST_FAILED, 0) + get(ST_ERROR, 0)
 
 
 def _target_classes(targets):
@@ -869,15 +887,19 @@ class Reporter(object):
     def exit_testclass (self, testclass, method_name, exc_info): pass
     def enter_testcase (self, testcase, testname): pass
     def exit_testcase  (self, testcase, testname, status, exc_info): pass
+    def enter_testcontext (self, context): pass
+    def exit_testcontext  (self, context): pass
 
 
 class BaseReporter(Reporter):
 
     INDICATOR = {
-        ST_PASSED:  "ok",          # or "passed" ?
+        ST_PASSED:  "passed",          # or "ok" ?
         ST_FAILED:  "Failed",
         ST_ERROR:   "ERROR",
         ST_SKIPPED: "skipped",
+        ST_TODO:    "TODO",
+        #ST_UNEXPECTED: "Unexpected",
     }
 
     separator =  "-" * 70
@@ -886,6 +908,7 @@ class BaseReporter(Reporter):
         self._color = color
         self.out = out
         self.counts = {}
+        self._context_stack = []
 
     def _set_color(self, color=None):
         if color is not None:
@@ -910,17 +933,33 @@ class BaseReporter(Reporter):
     out = property(__get_out, __set_out)
 
     def clear_counts(self):
-        self.counts = {ST_PASSED: 0, ST_FAILED: 0, ST_ERROR: 0, ST_SKIPPED: 0}
+        self.counts = {
+            ST_PASSED:     0,
+            ST_FAILED:     0,
+            ST_ERROR:      0,
+            ST_SKIPPED:    0,
+            ST_TODO:       0,
+            #ST_UNEXPECTED: 0,
+        }
+
+    _counts2str_table = [
+        (ST_PASSED,     "passed",     True),
+        (ST_FAILED,     "failed",     True),
+        (ST_ERROR,      "error",      True),
+        (ST_SKIPPED,    "skipped",    True),
+        (ST_TODO,       "todo",       True),
+        #(ST_UNEXPECTED, "unexpected", False),
+    ]
 
     def counts2str(self):
         buf = [None]; add = buf.append
         total = 0
-        for word, status in zip(("passed", "failed", "error", "skipped"),
-                                (ST_PASSED, ST_FAILED, ST_ERROR, ST_SKIPPED)):
+        for word, status, required in self._counts2str_table:
             n = self.counts.get(status, 0)
             s = "%s:%s" % (word, n)
             if n: s = self.colorize(s, status)
-            add(s)
+            if required or n:
+                add(s)
             total += n
         buf[0] = "total:%s" % total
         return ", ".join(buf)
@@ -934,7 +973,7 @@ class BaseReporter(Reporter):
         min = int(int(dt) / 60)     # int / int is float on Python3
         sec = dt - (min * 60)
         elapsed = min and "%s:%06.3f" % (min, sec) or "%.3f" % sec
-        self.out.write("## %s   (elapsed %s)\n" % (self.counts2str(), elapsed))
+        self.out.write("## %s  (%s sec)\n" % (self.counts2str(), elapsed))
         self.out.flush()
 
     def enter_testclass(self, testclass):
@@ -944,7 +983,7 @@ class BaseReporter(Reporter):
         for tupl in self._exceptions:
             self.report_exceptions(*tupl)
         if exc_info:
-            self.report_exception(testclass, method_name, ST_ERROR, exc_info)
+            self.report_exception(testclass, method_name, ST_ERROR, exc_info, None)
         if self._exceptions or exc_info:
             self.write_separator()
         self.out.flush()
@@ -954,8 +993,16 @@ class BaseReporter(Reporter):
 
     def exit_testcase(self, testcase, testname, status, exc_info):
         self.counts[status] = self.counts.setdefault(status, 0) + 1
-        if exc_info:
-            self._exceptions.append((testcase, testname, status, exc_info))
+        if exc_info and status != ST_SKIPPED:
+            context = self._context_stack and self._context_stack[-1] or None
+            self._exceptions.append((testcase, testname, status, exc_info, context))
+
+    def enter_testcontext(self, context):
+        self._context_stack.append(context)
+
+    def exit_testcontext(self, context):
+        popped = self._context_stack.pop()
+        assert popped is context
 
     def indicator(self, status):
         indicator = self.INDICATOR.get(status) or '???'
@@ -971,30 +1018,37 @@ class BaseReporter(Reporter):
         meth = getattr(testcase, testname)
         return meth and meth.__doc__ and meth.__doc__ or testname
 
-    def report_exceptions(self, testcase, testname, status, exc_info):
+    def report_exceptions(self, testcase, testname, status, exc_info, context):
         if isinstance(exc_info, list):
             specs = exc_info
             for spec in specs:
-                self.report_spec_esception(testcase, testname, status, spec)
+                self.report_spec_esception(testcase, testname, status, spec, context)
         else:
-            self.report_exception(testcase, testname, status, exc_info)
+            self.report_exception(testcase, testname, status, exc_info, context)
 
-    def report_exception(self, testcase, testname, status, exc_info):
-        self.report_exception_header(testcase, testname, status, exc_info)
-        self.report_exception_body  (testcase, testname, status, exc_info)
-        self.report_exception_footer(testcase, testname, status, exc_info)
+    def report_exception(self, testcase, testname, status, exc_info, context):
+        self.report_exception_header(testcase, testname, status, exc_info, context)
+        self.report_exception_body  (testcase, testname, status, exc_info, context)
+        self.report_exception_footer(testcase, testname, status, exc_info, context)
 
-    def report_exception_header(self, testcase, testname, status, exc_info):
+    def report_exception_header(self, testcase, testname, status, exc_info, context):
         if isinstance(testcase, type):
             klass, method = testcase, testname
-            parent = self.get_testclass_name(klass)
-            child  = method + '()'
+            title = "%s > %s()" % (self.get_testclass_name(klass), method)
             desc   = None
         else:
             parent, child, desc = self._get_testcase_header_items(testcase, testname)
+            items = [child]
+            c = context
+            while c:
+                items.append(c.desc)
+                c = c.parent
+            items.append(parent)
+            items.reverse()
+            title = " > ".join(items)
         indicator = self.indicator(status)
         self.write_separator()
-        self.out.write("[%s] %s > %s\n" % (indicator, parent, child))
+        self.out.write("[%s] %s\n" % (indicator, title))
         if desc: self.out.write(desc + "\n")
 
     def _get_testcase_header_items(self, testcase, testname):
@@ -1011,7 +1065,7 @@ class BaseReporter(Reporter):
         #return not filename.startswith(_oktest_filepath)
         return "__unittest" not in tb.tb_frame.f_globals
 
-    def report_exception_body(self, testcase, testname, status, exc_info):
+    def report_exception_body(self, testcase, testname, status, exc_info, context):
         assert exc_info
         ex_class, ex, ex_traceback = exc_info
         filter = not config.debug and self._filter or None
@@ -1030,26 +1084,28 @@ class BaseReporter(Reporter):
             if not rest.endswith("\n"): self.out.write("\n")
         self.out.flush()
 
-    def report_exception_footer(self, testcase, testname, status, exc_info):
+    def report_exception_footer(self, testcase, testname, status, exc_info, context):
         pass
 
     def _print_temporary_str(self, string):
         if is_tty(self.out):
-            self.__string = string
+            #self.__string = string
             self.out.write(string)
             self.out.flush()
 
-    def _erase_temporary_str(self):
+    def _erase_temporary_str(self, _eraser="\b"*255):
         if is_tty(self.out):
-            n = len(self.__string) + 1    # why '+1' ?
-            self.out.write("\b" * n)
+            #n = len(self.__string) + 1    # why '+1' ?
+            #self.out.write("\b" * n)      # not work with wide-chars
+            #self.out.flush()
+            #del self.__string
+            self.out.write(_eraser)
             self.out.flush()
-            del self.__string
 
-    def report_spec_esception(self, testcase, testname, status, spec):
+    def report_spec_esception(self, testcase, testname, status, spec, context):
         ex = spec._exception
         exc_info = (ex.__class__, ex, spec._traceback)
-        #self.report_exception_header(testcase, testname, status, exc_info)
+        #self.report_exception_header(testcase, testname, status, exc_info, context)
         parent, child, desc = self._get_testcase_header_items(testcase, testname)
         indicator = self.indicator(status)
         self.write_separator()
@@ -1059,8 +1115,8 @@ class BaseReporter(Reporter):
         stacktrace = self._filter_stacktrace(spec._stacktrace, spec._traceback)
         self._print_stacktrace(stacktrace)
         #
-        self.report_exception_body(testcase, testname, status, exc_info)
-        self.report_exception_footer(testcase, testname, status, exc_info)
+        self.report_exception_body(testcase, testname, status, exc_info, context)
+        self.report_exception_footer(testcase, testname, status, exc_info, context)
 
     def _filter_stacktrace(self, stacktrace, traceback_):
         entries = traceback.extract_tb(traceback_)
@@ -1081,15 +1137,18 @@ class BaseReporter(Reporter):
             self.out.write('  File "%s", line %s, in %s\n' % (file, line, func))
             self.out.write('    %s\n' % text)
 
-    def colorize(self, string, status):
+    def colorize(self, string, kind):
         if not self._color:
             return string
-        if status == ST_PASSED:  return Color.green(string, bold=True)
-        if status == ST_FAILED:  return Color.red(string, bold=True)
-        if status == ST_ERROR:   return Color.red(string, bold=True)
-        if status == ST_SKIPPED: return Color.yellow(string, bold=True)
-        if status == "topic":    return Color.bold(string)
-        if status == "sep":      return Color.red(string)
+        if kind == ST_PASSED:  return Color.green(string, bold=True)
+        if kind == ST_FAILED:  return Color.red(string, bold=True)
+        if kind == ST_ERROR:   return Color.red(string, bold=True)
+        if kind == ST_SKIPPED: return Color.yellow(string, bold=True)
+        if kind == ST_TODO:    return Color.yellow(string, bold=True)
+        #if kind == ST_UNEXPECTED: return Color.red(string, bold=True)
+        if kind == "topic":    return Color.bold(string)
+        if kind == "sep":      return Color.red(string)
+        if kind == "context":  return Color.bold(string)
         return Color.yellow(string)
 
     def write_separator(self):
@@ -1102,6 +1161,8 @@ class BaseReporter(Reporter):
                 ST_FAILED : self.colorize("f", ST_FAILED ),
                 ST_ERROR  : self.colorize("E", ST_ERROR  ),
                 ST_SKIPPED: self.colorize("s", ST_SKIPPED),
+                ST_TODO   : self.colorize("t", ST_TODO),
+                #ST_UNEXPECTED: self.colorize("u", ST_UNEXPECTED),
                 None      : self.colorize("?", None),
             }
         return self._status_chars.get(status) or self._status_chars.get(None)
@@ -1162,6 +1223,7 @@ class VerboseReporter(BaseReporter):
 
     def __init__(self, *args, **kwargs):
         self._super.__init__(self, *args, **kwargs)
+        self.depth = 1
 
     def enter_testclass(self, testclass):
         self._super.enter_testclass(self, testclass)
@@ -1170,16 +1232,35 @@ class VerboseReporter(BaseReporter):
 
     def enter_testcase(self, testcase, testname):
         desc = self.get_testcase_desc(testcase, testname)
-        self._print_temporary_str("  - [  ] " + desc)
+        self._print_temporary_str("  " * self.depth + "- [      ] " + desc)
 
     def exit_testcase(self, testcase, testname, status, exc_info):
+        s = ""
+        if status == ST_SKIPPED:
+            ex = exc_info[1]
+            #reason = getattr(ex, 'reason', '')
+            reason = ex.args[0]
+            s = " (reason: %s)" % (reason, )
+            exc_info = ()
         self._super.exit_testcase(self, testcase, testname, status, exc_info)
         self._erase_temporary_str()
         indicator = self.indicator(status)
         desc = self.get_testcase_desc(testcase, testname)
-        self.out.write("  - [%s] %s\n" % (indicator, desc))
+        self.out.write("  " * self.depth + "- [%s] %s%s\n" % (indicator, desc, s))
         self.out.flush()
 
+    def enter_testcontext(self, context):
+        self._super.enter_testcontext(self, context)
+        s = context.desc
+        if not (s.startswith("when ") or s == "else:"):
+            s = self.colorize(s, "context")
+        self.out.write("  " * self.depth + "+ %s\n" % s)
+        self.depth += 1
+
+    def exit_testcontext(self, context):
+        self._super.exit_testcontext(self, context)
+        self.depth -= 1
+
 BaseReporter.register_class("verbose", VerboseReporter)
 
 
@@ -1262,7 +1343,7 @@ class UnittestStyleReporter(BaseReporter):
             self.report_exceptions(*tupl)
         self._super.exit_all(self)
 
-    def report_exception_header(self, testcase, testname, status, exc_info):
+    def report_exception_header(self, testcase, testname, status, exc_info, context):
         if isinstance(testcase, type):
             klass, method = testcase, testname
             parent = self.get_testclass_name(klass)
@@ -1321,6 +1402,12 @@ class OldStyleReporter(BaseReporter):
                 self.out.write(x)
         elif status == ST_SKIPPED:
             self.out.write("[skipped]\n")
+        elif status == ST_TODO:
+            self.out.write("[TODO]\n")
+        #elif status == ST_UNEXPECTED:
+        #    self.out.write("[Unexpected]\n")
+        else:
+            assert False, "UNREACHABLE: status=%r" % (status,)
 
 BaseReporter.register_class("oldstyle", SimpleReporter)
 
@@ -1415,9 +1502,9 @@ class _RunnableContext(_Context):
 
 
 ##
-## spec()
+## spec()   # deprecated
 ##
-class Spec(_Context):
+class Spec(_Context):   # deprecated
 
     _exception  = None
     _traceback  = None
@@ -1479,7 +1566,10 @@ class Spec(_Context):
     __nonzero__ = __bool__    # for Python2
 
 
-def spec(desc):
+def spec(desc):   # deprecated
+    if not os.getenv('OKTEST_WARNING_DISABLED'):
+        import warnings
+        warnings.warn("oktest.spec() is deprecated.", DeprecationWarning, 2)
     return Spec(desc)
 
 
@@ -1494,7 +1584,7 @@ def test(description_text=None, **options):
     n = localvars.get('__n', 0) + 1
     localvars['__n'] = n
     def deco(orig_func):
-        argnames = func_argnames(orig_func)
+        argnames = util.func_argnames(orig_func)
         fixture_names = argnames[1:]   # except 'self'
         if fixture_names:
             def newfunc(self):
@@ -1542,10 +1632,10 @@ class FixtureInjector(object):
         resolved  = {"self": object}   # {"arg_name": arg_value}
         in_progress = []
         ##
-        arg_names = func_argnames(func)
+        arg_names = util.func_argnames(func)
         ## default arg values of test method are stored into 'resolved' dict
         ## in order for providers to access to them
-        defaults = func_defaults(func)
+        defaults = util.func_defaults(func)
         if defaults:
             idx = - len(defaults)
             for aname, default in zip(arg_names[idx:], defaults):
@@ -1564,11 +1654,11 @@ class FixtureInjector(object):
                     resolved[aname] = fixture_manager.provide(aname)
             return resolved[aname]
         def _call(provider, resolving_arg_name):
-            arg_names = func_argnames(provider)
+            arg_names = util.func_argnames(provider)
             if not arg_names:
                 return provider()
             in_progress.append(resolving_arg_name)
-            defaults = func_defaults(provider)
+            defaults = util.func_defaults(provider)
             if not defaults:
                 arg_values = [ _get_value(aname) for aname in arg_names ]
             else:
@@ -1643,7 +1733,123 @@ class LoopedDependencyError(ValueError):
 
 
 ##
-## helpers
+## test context
+##
+def context():
+
+    __all__ = ('subject', 'situation', )
+    global TestContext
+
+    class TestContext(object):
+        """grouping test methods.
+
+        normally created with subject() or situation() helpers.
+
+        ex::
+            class HelloClassTest(unittest.TestCase):
+                SUBJECT = Hello
+                with subject('#method1()'):
+                    @test("spec1")
+                    def _(self):
+                        ...
+                    @test("spec2")
+                    def _(self):
+                        ...
+                with subject('#method2()'):
+                    with situation('when condition:'):
+                        @test("spec3")
+                        def _(self):
+                    with situation('else:')
+                        @test("spec3")
+                        def _(self):
+                        ...
+        """
+
+        def __init__(self, desc, _lineno=None):
+            self.desc = desc
+            self.items = []
+            self.parent = None
+            self._lineno = _lineno
+
+        def __repr__(self):
+            return "<TestContext desc=%r items=[%s]>" % \
+                       (self.desc, ','.join(repr(x) for x in self.items))
+
+        def __enter__(self):
+            f_locals = sys._getframe(1).f_locals
+            self._f_locals = f_locals
+            self._varnames = set(f_locals.keys())
+            stack = f_locals.setdefault('_context_stack', [])
+            if not stack:
+                f_locals.setdefault('_context_list', []).append(self)
+            else:
+                self.parent = stack[-1]
+                self.parent.items.append(self)
+            stack.append(self)
+            return self
+
+        def __exit__(self, *args):
+            f_locals = self._f_locals
+            popped = f_locals['_context_stack'].pop()
+            assert popped is self
+            newvars = set(f_locals.keys()) - self._varnames
+            for name in newvars:
+                if name.startswith('test'):
+                    func = f_locals[name]
+                    if not hasattr(func, '_test_context'):
+                        func._test_context = self.desc
+                        self.items.append((name, func))
+            self._sort_items(self.items)
+            del self._f_locals
+            del self._varnames
+
+        @staticmethod
+        def _sort_items(items):
+            def fn(item):
+                if isinstance(item, tuple):
+                    return getattr(item[1], '_firstlineno', None) or \
+                           _func_firstlineno(item[1])
+                elif isinstance(item, TestContext):
+                    return item._lineno or 0
+                else:
+                    assert False, "** item=%r" % (item, )
+            items.sort(key=fn)
+
+        @staticmethod
+        def _inspect_items(items):
+            def _inspect(items, depth, add):
+                for item in items:
+                    if isinstance(item, tuple):
+                        add("  " * depth + "- %s()\n" % item[0])
+                    else:
+                        add("  " * depth + "- Context: %r\n" % item.desc)
+                        _inspect(item.items, depth+1, add)
+            buf = []
+            _inspect(items, 0, buf.append)
+            return "".join(buf)
+
+
+    def subject(desc):
+        """helper to group test methods by subject"""
+        lineno = sys._getframe(1).f_lineno
+        return TestContext(desc, _lineno=lineno)
+
+    def situation(desc):
+        """helper to group test methods by situation or condition"""
+        lineno = sys._getframe(1).f_lineno
+        return TestContext(desc, _lineno=lineno)
+
+
+    return locals()
+
+context = _new_module("oktest.context", context())
+context.TestContext = TestContext
+subject   = context.subject
+situation = context.situation
+
+
+##
+## util
 ##
 def _dummy():
 
@@ -1717,13 +1923,77 @@ def _dummy():
                 from shutil import rmtree
                 rmtree(fname)
 
+    def get_location(depth=0):
+        frame = sys._getframe(depth+1)
+        return (frame.f_code.co_filename, frame.f_lineno)
+
+    def read_binary_file(fname):
+        f = open(fname, 'rb')
+        try:
+            b = f.read()
+        finally:
+            f.close()
+        return b
+
+    if python2:
+        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)')
+        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
+            b = _read_binary_file(fname)
+            m = _rexp.match(b)
+            encoding = m and m.group(1) or 'utf-8'
+            u = b.decode(encoding)
+            assert isinstance(u, unicode)
+            return u
+    if python3:
+        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)'.encode('us-ascii'))
+        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
+            b = _read_binary_file(fname)
+            m = _rexp.match(b)
+            encoding = m and m.group(1).decode('us-ascii') or 'utf-8'
+            u = b.decode(encoding)
+            assert isinstance(u, str)
+            return u
+
+    from types import MethodType as _MethodType
+
+    if python2:
+        def func_argnames(func):
+            if isinstance(func, _MethodType):
+                codeobj = func.im_func.func_code
+                index = 1
+            else:
+                codeobj = func.func_code
+                index = 0
+            return codeobj.co_varnames[index:codeobj.co_argcount]
+        def func_defaults(func):
+            if isinstance(func, _MethodType):
+                return func.im_func.func_defaults
+            else:
+                return func.func_defaults
+    if python3:
+        def func_argnames(func):
+            if isinstance(func, _MethodType):
+                codeobj = func.__func__.__code__
+                index = 1
+            else:
+                codeobj = func.__code__
+                index = 0
+            return codeobj.co_varnames[index:codeobj.co_argcount]
+        def func_defaults(func):
+            if isinstance(func, _MethodType):
+                return func.__func__.__defaults__
+            else:
+                return func.__defaults__
 
     return locals()
 
 
-helper = _new_module('oktest.helper', _dummy())
+util = _new_module('oktest.util', _dummy())
 del _dummy
 
+helper = util  ## 'help' is an alias of 'util' (for backward compatibility)
+sys.modules['oktest.helper'] = sys.modules['oktest.util']
+
 
 
 ##
@@ -1852,7 +2122,7 @@ def _dummy():
     return locals()
 
 
-dummy = _new_module('oktest.dummy', _dummy(), helper)
+dummy = _new_module('oktest.dummy', _dummy(), util)
 del _dummy
 
 
@@ -2067,13 +2337,13 @@ def _dummy():
     return locals()
 
 
-tracer = _new_module('oktest.tracer', _dummy(), helper)
+tracer = _new_module('oktest.tracer', _dummy(), util)
 del _dummy
 
 
 
 ##
-## main
+## mainapp
 ##
 
 def load_module(mod_name, filepath, content=None):
@@ -2083,9 +2353,9 @@ def load_module(mod_name, filepath, content=None):
     #mod.__dict__["__file__"] = os.path.abspath(filepath)
     if content is None:
         if python2:
-            content = _read_binary_file(filepath)
+            content = util.read_binary_file(filepath)
         if python3:
-            content = _read_text_file(filepath)
+            content = util.read_text_file(filepath)
     if filepath:
         code = compile(content, filepath, "exec")
         exec(code, mod.__dict__, mod.__dict__)
@@ -2390,15 +2660,20 @@ def _dummy():
             #    sys.stderr.write("%s" % (ex, ))
             #    sys.exit(1)
             app = cls(sys_argv[0])
-            n_errors = app.run()
+            n_errors = app.run(sys_argv[1:])
             sys.exit(n_errors)
 
     return locals()
 
 
-main = _new_module('oktest.main', _dummy(), helper)
+mainapp = _new_module('oktest.mainapp', _dummy(), util)
 del _dummy
 
 
+def main(*args):
+    sys_argv = [__file__] + sys.argv + list(args)
+    mainapp.MainApp.main(sys_argv)
+
+
 if __name__ == '__main__':
-    main.MainApp.main()
+    mainapp.MainApp.main()
