@@ -1,5 +1,5 @@
 ###
-### $Release: 0.9.0 $
+### $Release: 0.10.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -10,7 +10,7 @@ import unittest
 
 import oktest
 from oktest import ok, NG, run, spec
-from oktest.helper import *
+from oktest.util import *
 from oktest.tracer import Tracer, Call, FakeObject
 
 try:
@@ -18,6 +18,8 @@ try:
 except ImportError:
     from io import StringIO
 
+os.environ['OKTEST_WARNING_DISABLED'] = 'true'
+
 
 class Call_TC(unittest.TestCase):
 
